#!/usr/bin/env python
#
# Copyright (C) 2014 Narf Industries <info@narfindustries.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#


# This deck should leak 4 consecutive flag page bytes: trigger indices [140, 141, 142, 143]

POV_DECK_BYTES = ['\x04','\x06','\x04','\x09','\x04','\x05',
'\x05','\x07','\x02','\x03','\x0a','\x07','\x03','\x02','\x08','\x09','\x02','\x0a','\x0a','\x05','\x04','\x06',
'\x06','\x05','\x04','\x08','\x06','\x08','\x05','\x05','\x06','\x07','\x05','\x06','\x05','\x03','\x09','\x02',
'\x08','\x08','\x06','\x04','\x09','\x03','\x05','\x09','\x0a','\x07','\x05','\x04','\x05','\x02','\x07','\x08',
'\x09','\x05','\x0a','\x03','\x05','\x03','\x09','\x07','\x07','\x0a','\x04','\x0a','\x06','\x07','\x0a','\x0a',
'\x02','\x02','\x03','\x03','\x04','\x03','\x06','\x02','\x05','\x04','\x03','\x02','\x02','\x04','\x08','\x08',
'\x09','\x08','\x07','\x07','\x05','\x0a','\x09','\x08','\x06','\x07','\x08','\x06','\x05','\x07','\x09','\x05',
'\x09','\x08','\x04','\x06','\x03','\x09','\x09','\x04','\x07','\x05','\x03','\x05','\x0a','\x0a','\x06','\x08',
'\x09','\x02','\x09','\x05','\x0a','\x0a','\x0a','\x02','\x05','\x06','\x04','\x07','\x08','\x0a','\x0a','\x02',
'\x05','\x03','\x04','\x08','\x07','\x08','\x07','\x07','\x07','\x02','\x07','\x05','\x02','\x03','\x03','\x05',
'\x08','\x09','\x05','\x07','\x04','\x05','\x07','\x02','\x05','\x03','\x06','\x09','\x08','\x07','\x0a','\x08',
'\x06','\x07','\x05','\x09','\x02','\x05','\x02','\x03','\x0a','\x04','\x07','\x02','\x04','\x06','\x02','\x08',
'\x06','\x07','\x03','\x08','\x02','\x07','\x07','\x07','\x08','\x03','\x06','\x0a','\x09','\x09','\x07','\x07',
'\x07','\x07','\x0a','\x09','\x09','\x04','\x05','\x05','\x09','\x02','\x05','\x07','\x09','\x08','\x0a','\x02',
'\x04','\x08','\x0a','\x0a','\x0a','\x03','\x0a','\x02','\x04','\x07','\x08','\x0a','\x03','\x03','\x0a','\x06',
'\x03','\x06','\x05','\x06','\x09','\x02','\x02','\x02','\x07','\x07','\x06','\x08','\x05','\x02','\x09','\x09',
'\x0a','\x07','\x02','\x0a','\x08','\x05','\x05','\x02','\x02','\x02','\x09','\x0a','\x07','\x04','\x04','\x03',
'\x05','\x04','\x0a','\x02','\x05','\x0a','\x09','\x08','\x09','\x05','\x06','\x09','\x06','\x08','\x07','\x08',
'\x04','\x04','\x05','\x03','\x07','\x03','\x02','\x03','\x08','\x04','\x04','\x06','\x02','\x09','\x03','\x0a',
'\x05','\x02','\x09','\x04','\x03','\x07','\x03','\x08','\x03','\x03','\x08','\x0a','\x08','\x03','\x03','\x03',
'\x08','\x07','\x09','\x03','\x03','\x05','\x03','\x09','\x0a','\x07','\x0a','\x05','\x02','\x0a','\x07','\x03',
'\x0a','\x02','\x09','\x06','\x03','\x04','\x06','\x0a','\x02','\x04','\x05','\x0a','\x09','\x07','\x09','\x07',
'\x05','\x03','\x04','\x02','\x09','\x04','\x03','\x03','\x03','\x08','\x0a','\x05','\x0a','\x05','\x08','\x02',
'\x03','\x05','\x09','\x07','\x06','\x07','\x06','\x08','\x02','\x02','\x07','\x04','\x0a','\x07','\x04','\x05',
'\x0a','\x05','\x04','\x06','\x04','\x06','\x07','\x06','\x0a','\x02','\x05','\x04','\x02','\x04','\x0a','\x02',
'\x05','\x0a','\x08','\x07','\x02','\x09','\x06','\x05','\x08','\x0a','\x08','\x09','\x06','\x04','\x0a','\x03',
'\x09','\x08','\x0a','\x02','\x09','\x02','\x08','\x06','\x06','\x06','\x0a','\x04','\x05','\x09','\x08','\x05',
'\x09','\x0a','\x02','\x0a','\x03','\x0a','\x0a','\x03','\x06','\x0a','\x04','\x04','\x0a','\x06','\x07','\x0a',
'\x03','\x03','\x0a','\x09','\x04','\x03','\x05','\x06','\x08','\x0a','\x02','\x04','\x07','\x07','\x02','\x07',
'\x02','\x02','\x07','\x09','\x07','\x07','\x08','\x0a','\x02','\x07','\x08','\x07','\x0a','\x0a','\x04','\x03',
'\x02','\x04','\x02','\x02','\x04','\x04','\x02','\x07','\x03','\x05','\x05','\x08','\x07','\x02','\x03','\x02',
'\x09','\x07','\x0a','\x09','\x0a','\x04','\x07','\x06','\x0a','\x04','\x03','\x09','\x09','\x03','\x07','\x02',
'\x02','\x02','\x08','\x06','\x07','\x08','\x04','\x08','\x02','\x03','\x05','\x07','\x0a','\x08','\x03','\x05',
'\x02','\x04','\x04','\x03','\x08','\x0a','\x03','\x07','\x06','\x03','\x07','\x07','\x08','\x08','\x03','\x07',
'\x07','\x03','\x08','\x08','\x06','\x02','\x09','\x08','\x08','\x09','\x0a','\x0a','\x07','\x02','\x08','\x07',
'\x07','\x03','\x04','\x08','\x06','\x09','\x08','\x07','\x07','\x07','\x08','\x0a','\x04','\x05','\x06','\x07',
'\x06','\x0a','\x02','\x07','\x03','\x04','\x07','\x05','\x03','\x06','\x09','\x03','\x03','\x02','\x02','\x08',
'\x08','\x06','\x04','\x07','\x06','\x07','\x02','\x06','\x06','\x04','\x03','\x07','\x02','\x06','\x08','\x0a',
'\x02','\x08','\x04','\x03','\x05','\x04','\x0a','\x09','\x08','\x03','\x02','\x09','\x0a','\x02','\x0a','\x03',
'\x03','\x07','\x07','\x09','\x08','\x02','\x0a','\x04','\x06','\x0a','\x06','\x04','\x03','\x09','\x02','\x05',
'\x07','\x07','\x07','\x03','\x08','\x09','\x07','\x04','\x05','\x08','\x0a','\x02','\x03','\x02','\x04','\x0a',
'\x06','\x0a','\x0a','\x08','\x07','\x0a','\x05','\x08','\x05','\x07','\x02','\x08','\x06','\x0a','\x04','\x06',
'\x07','\x06','\x07','\x05','\x03','\x0a','\x05','\x0a','\x0a','\x0a','\x03','\x04','\x04','\x06','\x02','\x02',
'\x09','\x05','\x09','\x03','\x07','\x06','\x03','\x03','\x05','\x05','\x03','\x08','\x0a','\x03','\x08','\x05',
'\x0a','\x06','\x07','\x06','\x02','\x06','\x03','\x05','\x04','\x03','\x06','\x07','\x09','\x08','\x02','\x02',
'\x08','\x06','\x03','\x03','\x07','\x09','\x02','\x05','\x03','\x0a','\x04','\x08','\x03','\x06','\x03','\x02',
'\x09','\x02','\x0a','\x09','\x07','\x04','\x06','\x07','\x05','\x09','\x06','\x0a','\x06','\x05','\x05','\x06',
'\x0a','\x04','\x0a','\x03','\x0a','\x07','\x0a','\x08','\x04','\x05','\x07','\x03','\x0a','\x09','\x05','\x02',
'\x08','\x05','\x02','\x02','\x0a','\x08','\x0a','\x07','\x05','\x02','\x06','\x0a','\x0a','\x03','\x06','\x03',
'\x03','\x07','\x05','\x04','\x09','\x08','\x07','\x05','\x02','\x02','\x07','\x04','\x05','\x09','\x0a','\x09',
'\x08','\x07','\x06','\x02','\x08','\x04','\x06','\x08','\x04','\x07','\x03','\x06','\x07','\x09','\x02','\x08',
'\x0a','\x0a','\x05','\x08','\x03','\x06','\x06','\x05','\x06','\x05','\x05','\x08','\x06','\x09','\x04','\x08',
'\x05','\x04','\x04','\x07','\x06','\x05','\x02','\x05','\x07','\x08','\x03','\x09','\x09','\x04','\x0a','\x06',
'\x03','\x0a','\x02','\x07','\x07','\x04','\x02','\x03','\x09','\x08','\x02','\x09','\x03','\x09','\x0a','\x07',
'\x04','\x06','\x02','\x09','\x09','\x02','\x06','\x02','\x06','\x02','\x04','\x05','\x09','\x06','\x0a','\x02',
'\x0a','\x06','\x09','\x02','\x05','\x04','\x07','\x04','\x04','\x04','\x0a','\x05','\x0a','\x05','\x03','\x05',
'\x0a','\x08','\x05','\x08','\x08','\x08','\x0a','\x0a','\x09','\x09','\x08','\x08','\x03','\x0a','\x09','\x0a',
'\x03','\x08','\x07','\x08','\x09','\x05','\x02','\x03','\x0a','\x04','\x04','\x08','\x05','\x09','\x07','\x05',
'\x09','\x0a','\x09','\x06','\x08','\x06','\x02','\x08','\x0a','\x05','\x02','\x05','\x04','\x08','\x06','\x04',
'\x06','\x0a','\x06','\x0a','\x09','\x09','\x06','\x09','\x02','\x07','\x0a','\x05','\x09','\x03','\x06','\x05',
'\x07','\x08','\x02','\x09','\x08','\x08','\x03','\x04','\x04','\x06','\x09','\x07','\x07','\x03','\x08','\x09',
'\x07','\x09','\x04','\x03','\x06','\x0a','\x06','\x0a','\x03','\x09','\x0a','\x05','\x06','\x07','\x07','\x02',
'\x09','\x0a','\x07','\x0a','\x0a','\x02','\x0a','\x05','\x02','\x06','\x07','\x07','\x05','\x0a','\x09','\x07',
'\x0a','\x02','\x03','\x09','\x08','\x05','\x03','\x04','\x02','\x06','\x0a','\x08','\x09','\x06','\x0a','\x06',
'\x03','\x0a','\x0a','\x07','\x05','\x09','\x0a','\x04','\x08','\x05','\x09','\x0a','\x03','\x03','\x02','\x07',
'\x06','\x06','\x06','\x05','\x02','\x08','\x02','\x04','\x04','\x05','\x03','\x04','\x05','\x02','\x06','\x06',
'\x03','\x06','\x09','\x0a','\x04','\x02','\x03','\x03','\x07','\x07','\x06','\x05','\x03','\x03','\x03','\x03',
'\x0a','\x02','\x07','\x08','\x07','\x04','\x08','\x08','\x03','\x03','\x09','\x07','\x09','\x06','\x04','\x07',
'\x05','\x05','\x08','\x06','\x06','\x07','\x03','\x04','\x06','\x02','\x02','\x04','\x08','\x09','\x02','\x07',
'\x05','\x09','\x02','\x03','\x05','\x04','\x05','\x05','\x09','\x05','\x08','\x03','\x02','\x09','\x02','\x06',
'\x09','\x09','\x09','\x03','\x02','\x03','\x09','\x02','\x09','\x0a','\x07','\x08','\x03','\x06','\x08','\x08',
'\x05','\x07','\x02','\x08','\x09','\x0a','\x04','\x03','\x04','\x09','\x02','\x08','\x07','\x0a','\x07','\x03',
'\x02','\x04','\x06','\x0a','\x05','\x07','\x08','\x04','\x06','\x07','\x03','\x02','\x06','\x09','\x05','\x05',
'\x06','\x04','\x06','\x03','\x02','\x05','\x05','\x09','\x04','\x02','\x02','\x04','\x09','\x03','\x0a','\x0a',
'\x03','\x06','\x09','\x06','\x08','\x08','\x08','\x07','\x09','\x07','\x09','\x06','\x08','\x09','\x02','\x04',
'\x08','\x0a','\x06','\x09','\x03','\x06','\x06','\x09','\x05','\x06','\x0a','\x05','\x09','\x09','\x05','\x09',
'\x03','\x06','\x09','\x06','\x0a','\x03','\x05','\x06','\x05','\x05','\x05','\x07','\x07','\x05','\x02','\x03',
'\x02','\x07','\x04','\x07','\x03','\x08','\x05','\x05','\x05','\x0a','\x02','\x02','\x08','\x07','\x0a','\x04',
'\x09','\x09','\x08','\x07','\x06','\x04','\x03','\x06','\x08','\x02','\x0a','\x06','\x05','\x07','\x04','\x04',
'\x03','\x0a','\x09','\x07','\x0a','\x08','\x07','\x03','\x05','\x03','\x0a','\x0a','\x04','\x02','\x04','\x06',
'\x06','\x09','\x06','\x04','\x03','\x0a','\x08','\x03','\x03','\x06','\x02','\x0a','\x04','\x03','\x0a','\x03',
'\x09','\x08','\x03','\x03','\x0a','\x07','\x03','\x09','\x0a','\x05','\x04','\x08','\x08','\x04','\x04','\x08',
'\x06','\x04','\x05','\x08','\x06','\x07','\x04','\x03','\x09','\x02','\x08','\x07','\x05','\x08','\x09','\x08',
'\x02','\x05','\x04','\x0a','\x04','\x02','\x07','\x05','\x05','\x08','\x08','\x08','\x04','\x04','\x08','\x0a',
'\x02','\x04','\x03','\x0a','\x0a','\x08','\x08','\x08','\x08','\x04','\x05','\x08','\x09','\x04','\x05','\x02',
'\x09','\x06','\x07','\x03','\x09','\x09','\x09','\x03','\x04','\x0a','\x02','\x05','\x08','\x02','\x0a','\x07',
'\x0a','\x08','\x07','\x09','\x05','\x08','\x07','\x03','\x06','\x09','\x08','\x03','\x02','\x03','\x07','\x07',
'\x03','\x02','\x05','\x07','\x07','\x06','\x03','\x06','\x09','\x07','\x05','\x08','\x07','\x06','\x04','\x0a',
'\x09','\x09','\x04','\x07','\x0a','\x09','\x06','\x02','\x02','\x08','\x02','\x0a','\x03','\x0a','\x04','\x07',
'\x0a','\x08','\x0a','\x07','\x03','\x06','\x03','\x04','\x08','\x02','\x02','\x03','\x09','\x0a','\x09','\x04',
'\x03','\x09','\x02','\x02','\x03','\x05','\x04','\x04','\x08','\x02','\x07','\x03','\x05','\x0a','\x06','\x06',
'\x04','\x07','\x09','\x0a','\x05','\x09','\x09','\x08','\x0a','\x03','\x05','\x09','\x04','\x09','\x04','\x07',
'\x08','\x02','\x08','\x09','\x05','\x02','\x02','\x08','\x07','\x04','\x07','\x03','\x02','\x04','\x02','\x05',
'\x0a','\x08','\x08','\x09','\x02','\x07','\x08','\x05','\x08','\x03','\x04','\x06','\x0a','\x05','\x08','\x04',
'\x02','\x07','\x04','\x08','\x08','\x0a','\x05','\x07','\x03','\x04','\x03','\x0a','\x04','\x08','\x04','\x05',
'\x0a','\x02','\x09','\x05','\x05','\x07','\x08','\x04','\x02','\x05','\x07','\x07','\x09','\x04','\x02','\x07',
'\x07','\x02','\x04','\x07','\x09','\x08','\x04','\x03','\x0a','\x03','\x05','\x05','\x05','\x09','\x07','\x09',
'\x08','\x06','\x03','\x05','\x07','\x04','\x09','\x09','\x06','\x0a','\x03','\x0a','\x0a','\x09','\x02','\x04',
'\x06','\x08','\x04','\x05','\x03','\x09','\x05','\x05','\x03','\x05','\x07','\x02','\x0a','\x0a','\x02','\x06',
'\x09','\x08','\x05','\x09','\x05','\x07','\x06','\x03','\x06','\x06','\x05','\x03','\x04','\x06','\x0a','\x06',
'\x06','\x04','\x04','\x04','\x09','\x05','\x0a','\x02','\x04','\x06','\x07','\x07','\x05','\x07','\x05','\x09',
'\x04','\x07','\x09','\x06','\x09','\x09','\x0a','\x0a','\x03','\x06','\x03','\x03','\x08','\x0a','\x06','\x05',
'\x0a','\x09','\x03','\x03','\x08','\x03','\x02','\x02','\x08','\x02','\x0a','\x09','\x04','\x06','\x02','\x08',
'\x06','\x06','\x03','\x09','\x04','\x02','\x0a','\x02','\x03','\x08','\x05','\x04','\x0a','\x03','\x03','\x03',
'\x09','\x09','\x06','\x05','\x07','\x07','\x08','\x05','\x03','\x02','\x03','\x04','\x02','\x03','\x0a','\x09',
'\x09','\x04','\x07','\x08','\x02','\x02','\x05','\x09','\x03','\x0a','\x06','\x04','\x07','\x08','\x02','\x06',
'\x09','\x07','\x02','\x0a','\x03','\x03','\x07','\x08','\x05','\x08','\x02','\x08','\x04','\x05','\x04','\x09',
'\x08','\x08','\x08','\x02','\x08','\x06','\x03','\x0a','\x07','\x09','\x07','\x09','\x06','\x04','\x02','\x09',
'\x05','\x0a','\x02','\x09','\x0a','\x02','\x0a','\x02','\x09','\x0a','\x03','\x03','\x06','\x09','\x02','\x03',
'\x02','\x07','\x07','\x07','\x02','\x07','\x06','\x06','\x08','\x09','\x07','\x07','\x09','\x0a','\x08','\x06',
'\x09','\x07','\x06','\x02','\x04','\x09','\x03','\x04','\x03','\x04','\x06','\x0a','\x09','\x02','\x03','\x03',
'\x07','\x07','\x09','\x02','\x07','\x07','\x06','\x09','\x09','\x05','\x03','\x0a','\x03','\x0a','\x05','\x07',
'\x09','\x04','\x06','\x06','\x07','\x0a','\x09','\x03','\x03','\x06','\x08','\x04','\x08','\x07','\x09','\x02',
'\x09','\x09','\x05','\x02','\x04','\x02','\x0a','\x08','\x06','\x0a','\x0a','\x09','\x0a','\x06','\x05','\x0a',
'\x02','\x06','\x02','\x05','\x04','\x09','\x04','\x04','\x06','\x04','\x05','\x08','\x07','\x05','\x07','\x02',
'\x04','\x0a','\x05','\x0a','\x07','\x07','\x09','\x03','\x02','\x06','\x08','\x0a','\x02','\x05','\x0a','\x08',
'\x0a','\x09','\x0a','\x0a','\x08','\x02','\x02','\x04','\x03','\x06','\x03','\x09','\x06','\x0a','\x02','\x03',
'\x06','\x05','\x03','\x07','\x03','\x02','\x03','\x04','\x0a','\x04','\x02','\x06','\x04','\x03','\x08','\x09',
'\x08','\x08','\x04','\x08','\x06','\x05','\x09','\x09','\x0a','\x0a','\x08','\x03','\x0a','\x02','\x04','\x04',
'\x05','\x04','\x09','\x02','\x03','\x05','\x08','\x02','\x05','\x08','\x03','\x0a','\x07','\x07','\x02','\x08',
'\x02','\x02','\x03','\x0a','\x09','\x08','\x03','\x08','\x05','\x04','\x02','\x05','\x07','\x04','\x0a','\x04',
'\x06','\x02','\x06','\x08','\x08','\x08','\x0a','\x03','\x05','\x05','\x02','\x08','\x08','\x09','\x09','\x09',
'\x05','\x02','\x05','\x02','\x02','\x09','\x06','\x04','\x07','\x05','\x05','\x0a','\x04','\x06','\x07','\x08',
'\x0a','\x02','\x09','\x0a','\x03','\x07','\x0a','\x08','\x0a','\x04','\x09','\x09','\x02','\x04','\x06','\x08',
'\x09','\x05','\x06','\x03','\x02','\x06','\x08','\x06','\x02','\x02','\x07','\x07','\x08','\x02','\x06','\x04',
'\x02','\x08','\x03','\x02','\x04','\x05','\x05','\x09','\x09','\x03','\x08','\x0a','\x09','\x07','\x08','\x0a',
'\x03','\x07','\x06','\x03','\x02','\x05','\x09','\x08','\x05','\x02','\x09','\x07','\x0a','\x07','\x0a','\x08',
'\x0a','\x07','\x07','\x04','\x0a','\x02','\x05','\x04','\x06','\x03','\x07','\x03','\x03','\x04','\x06','\x05',
'\x04','\x04','\x0a','\x03','\x08','\x0a','\x0a','\x07','\x03','\x04','\x08','\x07','\x09','\x06','\x04','\x03',
'\x09','\x07','\x06','\x02','\x03','\x04','\x03','\x02','\x08','\x06','\x07','\x06','\x03','\x04','\x0a','\x09',
'\x05','\x09','\x05','\x06','\x0a','\x08','\x02','\x06','\x09','\x07','\x07','\x03','\x06','\x06','\x06','\x07',
'\x06','\x06','\x03','\x04','\x04','\x07','\x0a','\x02','\x08','\x09','\x07','\x08','\x08','\x03','\x08','\x09',
'\x0a','\x03','\x09','\x02','\x08','\x02','\x08','\x03','\x0a','\x02','\x03','\x05','\x04','\x05','\x02','\x05',
'\x04','\x09','\x05','\x09','\x02','\x08','\x03','\x03','\x09','\x02','\x07','\x02','\x05','\x0a','\x02','\x08',
'\x07','\x0a','\x08','\x05','\x04','\x03','\x03','\x02','\x07','\x02','\x04','\x09','\x03','\x06','\x03','\x06',
'\x07','\x09','\x03','\x07','\x09','\x02','\x02','\x02','\x04','\x06','\x06','\x07','\x02','\x08','\x02','\x02',
'\x05','\x08','\x03','\x03','\x06','\x0a','\x06','\x07','\x08','\x09','\x07','\x03','\x09','\x09','\x06','\x0a',
'\x04','\x02','\x04','\x05','\x07','\x07','\x02','\x07','\x08','\x03','\x04','\x03','\x03','\x05','\x05','\x08',
'\x07','\x05','\x06','\x0a','\x04','\x05','\x09','\x0a','\x02','\x08','\x08','\x05','\x02','\x06','\x0a','\x02',
'\x09','\x05','\x02','\x02','\x07','\x03','\x03','\x0a','\x02','\x09','\x09','\x03','\x06','\x07','\x03','\x03',
'\x02','\x04','\x04','\x0a','\x09','\x07','\x07','\x03','\x0a','\x03','\x04','\x03','\x03','\x06','\x06','\x06',
'\x04','\x02','\x03','\x06','\x04','\x05','\x0a','\x09','\x0a','\x08','\x09','\x05','\x0a','\x04','\x06','\x03',
'\x05','\x09','\x08','\x05','\x03','\x03','\x02','\x08','\x09','\x03','\x05','\x03','\x09','\x0a','\x07','\x05',
'\x02','\x0a','\x05','\x0a','\x06','\x09','\x09','\x04','\x09','\x0a','\x06','\x03','\x06','\x04','\x02','\x02',
'\x04','\x09','\x02','\x02','\x07','\x09','\x08','\x07','\x09','\x06','\x09','\x08','\x07','\x05','\x05','\x04',
'\x02','\x05','\x08','\x07','\x0a','\x04','\x06','\x07','\x09','\x07','\x04','\x05','\x03','\x03','\x02','\x06',
'\x03','\x08','\x05','\x06','\x0a','\x04','\x05','\x09','\x05','\x0a','\x04','\x08','\x09','\x05','\x0a','\x09',
'\x08','\x02','\x06','\x08','\x08','\x08','\x05','\x08','\x09','\x05','\x08','\x07','\x05','\x08','\x04','\x02',
'\x02','\x08','\x03','\x06','\x02','\x04','\x05','\x04','\x04','\x03','\x08','\x0a','\x0a','\x09','\x03','\x02',
'\x05','\x02','\x07','\x03','\x02','\x0a','\x03','\x05','\x06','\x08','\x03','\x07','\x06','\x03','\x09','\x04',
'\x03','\x04','\x05','\x0a','\x04','\x09','\x08','\x09','\x03','\x05','\x08','\x07','\x06','\x03','\x05','\x02',
'\x0a','\x04','\x0a','\x08','\x09','\x06','\x05','\x07','\x02','\x09','\x09','\x03','\x07','\x04','\x07','\x0a',
'\x0a','\x07','\x0a','\x02','\x08','\x03','\x07','\x06','\x08','\x05','\x02','\x07','\x09','\x05','\x02','\x03',
'\x02','\x06','\x06','\x08','\x0a','\x04','\x09','\x02','\x02','\x07','\x06','\x07','\x06','\x05','\x08','\x06',
'\x0a','\x05','\x04','\x08','\x06','\x04','\x06','\x02','\x09','\x06','\x08','\x02','\x06','\x07','\x03','\x06',
'\x02','\x03','\x09','\x05','\x06','\x04','\x02','\x08','\x09','\x02','\x06','\x08','\x08','\x0a','\x0a','\x04',
'\x02','\x0a','\x02','\x08','\x05','\x06','\x03','\x04','\x04','\x08','\x0a','\x06','\x03','\x05','\x05','\x03',
'\x08','\x07','\x08','\x04','\x04','\x08','\x07','\x05','\x02','\x03','\x09','\x0a','\x04','\x09','\x09','\x06',
'\x04','\x04','\x05','\x09','\x04','\x02','\x03','\x04','\x08','\x04','\x04','\x03','\x03','\x02','\x08','\x09',
'\x07','\x06','\x0a','\x02','\x03','\x07','\x02','\x05','\x02','\x07','\x08','\x04','\x02','\x04','\x06','\x05',
'\x07','\x05','\x03','\x08','\x07','\x07','\x09','\x09','\x0a','\x07','\x08','\x02','\x07','\x07','\x03','\x05',
'\x04','\x0a','\x0a','\x06','\x03','\x06','\x07','\x04','\x09','\x0a','\x06','\x0a','\x09','\x0a','\x07','\x04',
'\x03','\x07','\x04','\x04','\x0a','\x07','\x02','\x02','\x03','\x02','\x08','\x07','\x09','\x07','\x04','\x07',
'\x05','\x0a','\x09','\x05','\x07','\x03','\x07','\x03','\x05','\x02','\x06','\x08','\x08','\x06','\x07','\x04',
'\x06','\x08','\x04','\x09','\x03','\x09','\x05','\x05','\x05','\x07','\x03','\x0a','\x09','\x0a','\x05','\x04',
'\x06','\x08','\x08','\x02','\x04','\x09','\x04','\x06','\x04','\x08','\x02','\x05','\x02','\x09','\x08','\x0a',
'\x03','\x06','\x04','\x05','\x05','\x02','\x03','\x0a','\x0a','\x02','\x05','\x04','\x07','\x03','\x0a','\x05',
'\x07','\x04','\x0a','\x04','\x06','\x04','\x0a','\x05','\x05','\x02','\x08','\x04','\x08','\x08','\x05','\x05',
'\x05','\x08','\x09','\x0a','\x06','\x06','\x09','\x03','\x0a','\x06','\x03','\x07','\x09','\x04','\x07','\x04',
'\x03','\x03','\x08','\x08','\x08','\x06']