#!/usr/bin/env python

from generator.actions import Actions

import numpy as np
import random
import string

import os.path
here = os.path.dirname(os.path.realpath(__file__))

LOW = 1
GUARDED = 2
ELEVATED = 3
HIGH = 4
SEVERE = 5


def lts(l):
    if l == LOW:
        return 'LOW'
    if l == GUARDED:
        return 'GUARDED'
    if l == ELEVATED:
        return 'ELEVATED'
    if l == HIGH:
        return 'HIGH'
    if l == SEVERE:
        return 'SEVERE'

BASE_SIGNATURES = [
    {
        'Severity': LOW,
        'Path': "./test/sig1",
        'Data': "Gar Ma Nar Nar",
        'DataSize': 14
    },
    {
        'Severity': LOW,
        'Path': "./test/sig2",
        'Data': "Rick",
        'DataSize': 4
    },
    {
        'Severity': GUARDED,
        'Path': "./test/sig3",
        'Data': "Morty",
        'DataSize': 5
    },
    {
        'Severity': HIGH,
        'Path': "./test/sig4",
        'Data': "Summer",
        'DataSize': 6
    },
    {
        'Severity': SEVERE,
        'Path': "./test/sig5",
        'Data': "Krombopulous Michael",
        'DataSize': 20
    },
    {
        'Severity': LOW,
        'Path': "./test/sig6",
        'Data': "Bender",
        'DataSize': 6
    },
    {
        'Severity': LOW,
        'Path': "./test/sig7",
        'Data': "Planet Express",
        'DataSize': 14
    },
    {
        'Severity': LOW,
        'Path': "./test/sig8",
        'Data': "Gazorpazorpfield",
        'DataSize': 16
    },
    {
        'Severity': LOW,
        'Path': "./test/sig9",
        'Data': "Totally benign string",
        'DataSize': 21
    },
    {
        'Severity': LOW,
        'Path': "./test/sig10",
        'Data': "Evil string 23",
        'DataSize': 14
    },
    {
        'Severity': LOW,
        'Path': "./test/sig11",
        'Data': "Two Brothers",
        'DataSize': 12
    }
]


def rs(n, space=string.ascii_letters):
    return ''.join(random.choice(space) for _ in xrange(n))


def b2ul(s):
    h = np.uint32(5381)
    for c in s:
        c = np.uint8(ord(c))
        h = ((h << np.uint8(5)) + h) + c

    return h

class Signature(object):
    def __init__(self, path, data, data_size, severity):
        self.path = path
        self.data = data
        self.data_size = data_size
        self.severity = severity
        assert(self.data_size == len(self.data))

    @property
    def hash(self):
        return b2ul(self.data)


class Machine(Actions):
    def readln(self, s):
        self.read(expect=s, delim='\n')

    def writeln(self, s):
        self.write(str(s) + '\n')

    def start(self):
        #self.delay(100)
        self.state['signatures'] = []

        self.write(self.magic_page[:4])
        self.read(length=0x1000, expect=self.magic_page)

    def bridge1(self):
        pass

    def generate_signature(self):
        path = rs(12, string.ascii_letters + '/.')
        data = ' '.join(random.sample(WORDS, random.randint(1, 32)))
        data_size = len(data)
        severity = random.choice([
            LOW,
            GUARDED,
            ELEVATED,
            HIGH,
            SEVERE
        ])

        self.state['signatures'].append(Signature(path, data, data_size, severity))

    def add_signatures(self):
        self.writeln(len(self.state['signatures']))
        for signature in self.state['signatures']:
            self.writeln(signature.severity)
            self.writeln(len(signature.path))
            self.writeln(signature.path)
            self.writeln(signature.data_size)
            self.writeln(signature.data)
            self.readln("Added signature to database")

    def bridge2(self):
        pass

    def submit_sample(self):
        ds = ''
        rounds = random.randint(3, 20)

        for r in xrange(rounds):
            include_sig = self.chance(1.0 / rounds)

            if include_sig:
                ds += random.choice(self.state['signatures']).data + ' '
            else:
                ds += ' '.join(random.sample(WORDS, random.randint(1, 4))) + ' '

        self.writeln(len(ds))
        self.write(ds)

        matches = set()
        for signature in self.state['signatures']:
            if signature.data in ds:
                matches.add(signature)

        for match in sorted(matches, key=lambda s: s.path):
            self.readln(' - '.join([
                match.path,
                lts(match.severity),
                hex(match.hash)[2:len(hex(match.hash)) if hex(match.hash)[-1] != 'L' else len(hex(match.hash)) - 1]
            ]))

    def quit(self):
        self.writeln('0')

WORDS = [
"rubbernecku",
"lh",
"bantering",
"mailable",
"bregmatic",
"moneywort",
"meningitis",
"abridgment",
"whizbang",
"alkahestic",
"entbehr",
"unpursued",
"falsity",
"shawl",
"kahlua",
"colluctation",
"highflier",
"mimidae",
"kitchener",
"headway",
"crypt",
"irreproachably",
"tobagonian",
"multiplied",
"gatherum",
"husband",
"quiver",
"decent",
"longness",
"shipman",
"diodontidae",
"nfor",
"fate",
"kichai",
"airport",
"positive",
"primigenous",
"lahu",
"chlorococcum",
"avaler",
"cleveland",
"sward",
"squareness",
"disapprover",
"meniscium",
"invertebrate",
"precarious",
"cowlick",
"determinate",
"violently",
"supposing",
"porterhouse",
"languille",
"tape",
"behest",
"conjunctive",
"predisposed",
"carbomycin",
"burgee",
"millipede",
"adenocarcinoma",
"inebriation",
"dysdercus",
"unselfconsciousness",
"sigma",
"hallucinatory",
"flatwork",
"ute",
"whitewashed",
"pennyweight",
"carnegiea",
"handcar",
"crammer",
"uninflammable",
"popcorn",
"concentricity",
"guesthouse",
"apella",
"plasmodiidae",
"berber",
"transume",
"gatling",
"conch",
"tb",
"residual",
"macaronic",
"propylthiouracil",
"sitting",
"dyspepsia",
"sigmoidal",
"plasma",
"cut",
"tablinum",
"hemolytic",
"regional",
"senator",
"hobart",
"breasts",
"waw",
"recurrently",
"increase",
"noticed",
"demoralize",
"napalm",
"meanly",
"deformation",
"clandestine",
"pervicacy",
"unanimity",
"scootertrolley",
"oneirocritic",
"lunula",
"ascosporic",
"carbuncled",
"unproficiency",
"emulsified",
"ring",
"midmost",
"promptitude",
"humanist",
"abstention",
"gorgeous",
"suddenness",
"diskette",
"molest",
"noire",
"trippingly",
"kilogram",
"expunction",
"planscheme",
"fourpence",
"dicky",
"himalayan",
"flowerbed",
"deterration",
"squamata",
"inanimate",
"isobutylene",
"excess",
"racket",
"earwitness",
"morphallaxis",
"raster",
"warrior",
"csorcery",
"maximizing",
"unwillingness",
"indigo",
"natureal",
"grundy",
"scamper",
"gemma",
"edinburgh",
"concern",
"newcomer",
"auroroa",
"bisexuality",
"bouse",
"conclusively",
"publicly",
"callistephus",
"untunable",
"quintroon",
"skid",
"harlotry",
"cieceronian",
"software",
"adventurer",
"orthology",
"taguan",
"microelectronics",
"peacemaker",
"fitter",
"unimportant",
"ordering",
"embranglement",
"iraqi",
"flatly",
"asterisked",
"coceive",
"ventral",
"forgive",
"unfaceted",
"idiosyncratic",
"trailer",
"thunder",
"airhole",
"marching",
"criticise",
"foundered",
"handsome",
"medoc",
"mytilidae",
"disabling",
"deduce",
"gallope",
"utilizable",
"drosophilidae",
"enlil",
"reversioner",
"slopshop",
"talesman",
"plough",
"hyperphysical",
"capromyidae",
"coordinates",
"astute",
"singsong",
"inane",
"amplitudinous",
"athelstan",
"emetrol",
"setarious",
"eoraptor",
"iodoprotein",
"lain",
"flux",
"cheeseparing",
"bobtail",
"adoration",
"bikini",
"anaphor",
"longsuffering",
"taphephobia",
"illapse",
"sppiritual",
"fortis",
"charless",
"tarzan",
"alveolitis",
"patternmaker",
"earl",
"marks",
"intervenience",
"rationis",
"beneficiary",
"anointment",
"turtledoves",
"highroad",
"dejectedly",
"tether",
"sweatshop",
"sons",
"squireen",
"tropic",
"sukiyaki",
"incongruity",
"utensils",
"seraskier",
"windless",
"cogitate",
"currency",
"composure",
"mumpish",
"bacteriophagic",
"misterm",
"luge",
"craw",
"demigration",
"nepalese",
"arithmetical",
"relationships",
"cranny",
"expectoration",
"pedigree",
"whitecaps",
"happen",
"microphone",
"goldeneye",
"gulch",
"welladvised",
"beanie",
"sermonizer",
"appprove",
"meaningless",
"pupillari",
"polder",
"thickcoming",
"kinsman",
"haystack",
"executrix",
"dibs",
"typhoeus",
"rehabilitation",
"anthelmintic",
"decorativeness",
"neo",
"hagfish",
"raphus",
"glycine",
"hypoplasia",
"breechesmaker",
"sison",
"uncomfortable",
"etre",
"viceroyalty",
"fizz",
"foraging",
"fencible",
"roleplaying",
"recruits",
"repetita",
"hydrography",
"aerobe",
"coup",
"diabetic",
"kauai",
"hortatory",
"lustful",
"broussonetia",
"dispansion",
"bone",
"connotate",
"epact",
"data",
"foliated",
"minuscule",
"magically",
"corydalus",
"outstroke",
"holothuria",
"stannary",
"unwearied",
"adduce",
"reckon",
"oblong",
"quadrangle",
"fullflavored",
"fagin",
"synthetism",
"encroachment",
"abattoir",
"uncongeniality",
"rhinocerotidae",
"trainbearer",
"dapperling",
"faith",
"ride",
"nonbelligerent",
"droll",
"astropogon",
"foregolf",
"plunder",
"repellently",
"pedantry",
"heralded",
"legibility",
"button",
"isotopic",
"bedfellows",
"upscale",
"scooter",
"bulbed",
"personification",
"quito",
"helmholtz",
"myxine",
"libocedrus",
"secretion",
"morose",
"vibrator",
"scots",
"quinquefoliate",
"smocking",
"nymphalid",
"assemblage",
"qualms",
"perfice",
"protract",
"toboggan",
"cognoscible",
"pavlovian",
"aruspice",
"stages",
"liberate",
"ichthyologist",
"harshness",
"dong",
"tollitur",
"funera",
"brackish",
"memorize",
"bowhead",
"pelerine",
"animatism",
"counterbore",
"lawyerbush",
"merchant",
"indentation",
"coprolite",
"scalar",
"fluff",
"mini",
"prudery",
"nonrestrictive",
"fortran",
"bucephala",
"sensuality",
"celioma",
"unsubsantial",
"minikin",
"speed",
"psilotales",
"hydrochloride",
"yourself",
"exhaustible",
"proceeding",
"revocatory",
"fey",
"ascertainable",
"tides",
"undersize",
"egret",
"greasiness",
"stash",
"accipient",
"discoglossidae",
"kolkhoznik",
"ryukyuan",
"onomancy",
"grant",
"batter",
"unbarred",
"laparoscopy",
"haminoea",
"regulars",
"remora",
"vaccinium",
"espial",
"jobbery",
"gestalt",
"flabby",
"backwoods",
"best",
"lugubriously",
"lex",
"countershot",
"horseman",
"cshrillness",
"taxaceae",
"rankine",
"chrysology",
"monkish",
"maturely",
"milan",
"presumably",
"skinhead",
"opinionatedness",
"festoon",
"pelvis",
"sublimification",
"passetemps",
"roughish",
"clapping",
"presidential",
"grot",
"premium",
"stratiform",
"woolly",
"ruby",
"trecker",
"mb",
"laugh",
"viewgraph",
"accoutered",
"aditi",
"sticherus",
"cholecystitis",
"beldam",
"sew",
"unexplored",
"piddock",
"hearthstone",
"foremanship",
"formidable",
"murderer",
"incessant",
"suppose",
"machination",
"outpatient",
"mediatize",
"park",
"unbranded",
"excathedra",
"regnellidium",
"americanism",
"typed",
"mussulman",
"hellebore",
"confluence",
"interlink",
"devilmaycare",
"sopranino",
"morphology",
"serving",
"thickened",
"stylistically",
"inhaling",
"conspicuity",
"gladiate",
"gerres",
"aridity",
"headmistresship",
"stultification",
"recompense",
"torpescent",
"nearer",
"syncope",
"chloasma",
"jaunting",
"adience",
"coltish",
"undazzled",
"kordofan",
"trilliaceae",
"tantra",
"silverpoint",
"boards",
"polyphosphate",
"toper",
"apparel",
"angevin",
"ze",
"rhizophoraceae",
"levitical",
"camarade",
"clangula",
"hydrophobic",
"particularity",
"lampe",
"announced",
"solferino",
"pewter",
"condescending",
"argillite",
"wordcatcher",
"ridiculus",
"lamedh",
"linendraper",
"goodnatured",
"uncreated",
"hobsons",
"portion",
"virilis",
"galliass",
"chaser",
"fitted",
"staleness",
"scaffolding",
"cwater",
"buffle",
"conserve",
"phyllostomus",
"jocoseness",
"calash",
"latinate",
"lockmaster",
"fez",
"trill",
"liquidator",
"hydrargyrum",
"unexamined",
"chaffweed",
"incognita",
"agoraphobic",
"poundage",
"picturesque",
"summity",
"patrimony",
"brawl",
"economizer",
"insoluble",
"switchboard",
"pawn",
"averni",
"dullness",
"overmatch",
"roentgenray",
"autarchic",
"megalomaniacal",
"pyaemia",
"swagman",
"alchemic",
"chanted",
"exhumation",
"patriarchal",
"melampsora",
"introspective",
"viminaria",
"infrigidation",
"agentship",
"pharmacologist",
"timocracy",
"mist",
"lightfooted",
"unlabored",
"platyrrhine",
"hateful",
"faithfulness",
"payslip",
"playsuit",
"abandon",
"prawn",
"youthfully",
"coated",
"firmhold",
"pervicacity",
"unpriestly",
"micelle",
"phantom",
"ironweed",
"landscip",
"surly",
"neglectful",
"openwork",
"schoolmaster",
"aeneid",
"sonogram",
"admired",
"selectman",
"vegitous",
"inventiveness",
"infarction",
"cirrhosis",
"tubule",
"concussion",
"symphonizing",
"horseback",
"shorea",
"attaintment",
"hymnal",
"phraseology",
"chastening",
"hygrometer",
"oleaceae",
"ptarmigan",
"keep",
"promissory",
"unapologetic",
"anisometropic",
"whirring",
"catalog",
"deposit",
"apprehensiveness",
"diploma",
"tam",
"calembour",
"lycoperdon",
"viro",
"precipitously",
"norfolk",
"disceptation",
"freeman",
"actinomycetacaea",
"procellariidae",
"solitarily",
"indebtedness",
"resistance",
"tumbling",
"condemned",
"triptych",
"peccavi",
"rectified",
"fruiterer",
"columbo",
"conscionable",
"confidence",
"reagent",
"honored",
"roguishly",
"clammyweed",
"cats",
"coveting",
"shelving",
"acyclovir",
"rugged",
"autocratical",
"lull",
"winger",
"dipsomania",
"negatory",
"virtus",
"unappareled",
"leicestershire",
"terpene",
"nonfinancial",
"peru",
"numps",
"makes",
"theropod",
"updating",
"heth",
"uncinated",
"mess",
"bromic",
"reflex",
"lexicologist",
"castries",
"lagoon",
"alar",
"spot",
"thundertube",
"nostra",
"burgeon",
"pacatolus",
"ambiguous",
"circumbendibus",
"cycloloma",
"lese",
"causa",
"buckboard",
"tomahawk",
"ell",
"forecourt",
"spitsbergen",
"girth",
"accueil",
"martello",
"benghazi",
"corinth",
"basting",
"counterclaim",
"gropius",
"heritor",
"reprehensibility",
"innavigable",
"subarctic",
"fairly",
"impertubation",
"collembola",
"transsexual",
"export",
"bimorphemic",
"linguist",
"intruding",
"lasiocampa",
"showingcondemned",
"importunate",
"asclepiadaceous",
"impalpable",
"firmiana",
"rabelais",
"rancorous",
"splenitis",
"grantinaid",
"varuna",
"christened",
"weighbridge",
"suspend",
"roads",
"regimentals",
"und",
"hawthorn",
"formative",
"superimpose",
"extrapolation",
"annihilate",
"rainless",
"disinterestedly",
"impedient",
"uvulariaceae",
"culicidae",
"amphitheatric",
"profluence",
"promiscuously",
"duralumin",
"ineffectually",
"misproportion",
"rooibos",
"teff",
"turtle",
"amnestic",
"orbit",
"scallion",
"despoil",
"eighteenth",
"anticoagulation",
"pretrial",
"aphaeretic",
"wheeze",
"gage",
"consummate",
"pallette",
"flection",
"stilts",
"punk",
"confute",
"ingrained",
"anasa",
"cerecloth",
"psychopharmacology",
"prohibitively",
"snaggy",
"rattlebrained",
"cellophane",
"addax",
"ewer",
"pittidae",
"elapsed",
"quint",
"titubancy",
"tropically",
"beechen",
"frankincense",
"prefigure",
"graham",
"hybridoma",
"cardinalfish",
"hi",
"puttee",
"venetian",
"chordata",
"zygotene",
"vegan",
"rice",
"haemophilic",
"uninvented",
"purloo",
"toad",
"descension",
"holonym",
"parliamentarian",
"gingivitis",
"undamped",
"berliner",
"anthologist",
"clothesbrush",
"bauble",
"misinformation",
"intake",
"dike",
"protoarcheology",
"primed",
"hastyquick",
"conspectuity",
"incision",
"accurately",
"gadolinite",
"cosmos",
"lovers",
"argand",
"stigmatic",
"alterum",
"whean",
"slinky",
"bubo",
"hypo",
"tenez",
"epanaphora",
"convene",
"rehabilitative",
"storax",
"superincumbent",
"sinned",
"casuarinaceae",
"intricacy",
"bereft",
"gynecologist",
"precipitation",
"divagation",
"harpagon",
"aardvark",
"rhododendron",
"ld",
"tymbal",
"maxillaria",
"equivocate",
"chokey",
"lovely",
"propylon",
"discorporate",
"stagy",
"stag",
"cheeked",
"protean",
"quash",
"measurably",
"spinout",
"piperaceae",
"lowbrow",
"anthriscus",
"paralyze",
"bondslave",
"transportation",
"gilgamish",
"hydrolysis",
"teaberry",
"triplochiton",
"dredger",
"appealing",
"mannish",
"baldwin",
"adieu",
"rotunda",
"predatorial",
"nowise",
"aboriginal",
"sparrow",
"doomed",
"stole",
"melodist",
"vane",
"ascaridae",
"bunch",
"limicolae",
"flimsiness",
"esto",
"carnage",
"warehouse",
"nonadjacent",
"guereza",
"lampridae",
"agonus",
"unsearched",
"playa",
"simmer",
"coquet",
"flunkeyism",
"burdened",
"coincidence",
"halo",
"pinhole",
"mums",
"pyrometer",
"hotelier",
"placard",
"feeling",
"charity",
"benediction",
"calystegia",
"lipase",
"distaff",
"trademark",
"thalictrum",
"benedict",
"retrorse",
"rapprochement",
"superconductivity",
"bhutanese",
"entablature",
"demonic",
"perfoliate",
"penicillin",
"wafer",
"alcea",
"obtuseness",
"visaged",
"flirtation",
"coragyps",
"pandean",
"walleye",
"rid",
"backdrop",
"receptive",
"laterite",
"quinquennium",
"grayback",
"oil",
"amethystine",
"phenacomys",
"sudorific",
"ichiro",
"sergeant",
"zapus",
"supertitle",
"tody",
"canny",
"cheerlessly",
"antheraea",
"advocate",
"perforation",
"botanical",
"neutralized",
"defensor",
"untaught",
"lunation",
"pleasurableness",
"blanched",
"unheated",
"moliere",
"proviso",
"cloy",
"analogist",
"vacancy",
"forfeiture",
"colloid",
"veneering",
"loon",
"gavelock",
"surtout",
"schemer",
"triassic",
"puffinus",
"facia",
"defoliation",
"pureeyed",
"confessor",
"sweetsop",
"somersault",
"reason",
"gentamicin",
"misdoing",
"hospitaler",
"misbranded",
"zaire",
"crapulous",
"coraciidae",
"boehme",
"unassociated",
"unquestionable",
"baghdad",
"shine",
"penologist",
"dakota",
"skimmer",
"pseudolarix",
"aldosteronism",
"rhizopogonaceae",
"tourist",
"cuisinecordon",
"mediatorship",
"us",
"burglarious",
"coemption",
"nag",
"equanimity",
"tweeze",
"pekinese",
"consistence",
"bijoutry",
"exosmose",
"unergo",
"pedlar",
"pectoral",
"agonidae",
"bustard",
"downdraft",
"cozenage",
"multiracial",
"cakes",
"hippoglossus",
"nonextension",
"galbanum",
"schedule",
"amharic",
"glare",
"palinurus",
"phytoplankton",
"manhattan",
"amniote",
"cabana",
"sambar",
"ultracentrifugation",
"acadia",
"teflon",
"sculler",
"megalopolis",
"chad",
"teashop",
"adultery",
"csubstitute",
"bilabial",
"chelydra",
"stuffing",
"kaffir",
"multiplier",
"escheat",
"bodacious",
"diospyros",
"chlorambucil",
"delusory",
"invent",
"come",
"delusional",
"achromia",
"inculcate",
"panacea",
"glaciate",
"sempervirid",
"amphidiploid",
"precisely",
"ideographically",
"minelayer",
"squadrong",
"leontodon",
"unpriced",
"causes",
"modifiable",
"nonlethal",
"deux",
"glomeration",
"womanly",
"delirious",
"phrlike",
"nymphomaniac",
"wimpy",
"disable",
"pedodontist",
"formlessly",
"derange",
"belduque",
"known",
"cosmical",
"grotty",
"coherently",
"whistler",
"unsweetened",
"archidiskidon",
"triarchy",
"barnyard",
"ipsissimis",
"wickedness",
"fittings",
"prong",
"amateurish",
"serranidae",
"linguae",
"trogium",
"pricking",
"crisp",
"brand",
"corradiation",
"slovenian",
"red",
"nonpertinence",
"traveling",
"residentiary",
"contraposition",
"titanosauridae",
"litany",
"fritillaria",
"constituted",
"legem",
"sedate",
"gelatinous",
"perched",
"willis",
"compound",
"charivari",
"rightish",
"antimacassar",
"uncanonical",
"circadian",
"relevantly",
"belike",
"prairial",
"crustacea",
"foulard",
"averment",
"teammate",
"cashbox",
"deity",
"tetravalent",
"disburden",
"indiaman",
"subsurface",
"lackwit",
"straping",
"urubupunga",
"deviously",
"overjoyed",
"thill",
"bellboy",
"hindu",
"sleeper",
"moonshell",
"ceruse",
"mofussil",
"dimensioning",
"ascensional",
"carouser",
"barefaced",
"congenite",
"overshoe",
"castellan",
"naumachy",
"upbraiding",
"marines",
"anabaptist",
"stringybark",
"spazza",
"scapulohumeral",
"baker",
"bison",
"juris",
"untrained",
"alfilaria",
"birdie",
"cowardice",
"animalia",
"audacious",
"stoneblind",
"borne",
"sozzle",
"lancewood",
"tameness",
"paintable",
"ovulation",
"unwell",
"wk",
"pulcinella",
"forge",
"ly",
"mouthlike",
"epanalepsis",
"dishabille",
"leone",
"pooh",
"cobble",
"parallelism",
"fonctionnaire",
"dribbler",
"escrapment",
"enrolled",
"granulated",
"meruit",
"tupungato",
"omani",
"plasticity",
"mediant",
"adrenalectomy",
"selfcounsel",
"monster",
"exuberantly",
"hindi",
"sweep",
"unequipped",
"hazardous",
"ponderation",
"verbal",
"undergraduate",
"disincentive",
"little",
"undefeated",
"lissome",
"thallophytic",
"ametropia",
"phalaropidae",
"venturesome",
"limnology",
"sandbagger",
"nomology",
"sombrero",
"cephalopoda",
"administrator",
"carom",
"becalmed",
"ail",
"deprivation",
"superplus",
"pallid",
"rupert",
"socioeconomic",
"curiosity",
"undiminished",
"responsibly",
"triumphantly",
"virtuoso",
"lovingness",
"rights",
"hurdler",
"revetment",
"idyl",
"coplanar",
"patrol",
"scrappiness",
"dilapidate",
"abandoned",
"contrasty",
"handset",
"spuriously",
"austromancy",
"acanthocytosis",
"testament",
"demoniac",
"ardor",
"bilaterally",
"nonprehensile",
"arsenopyrite",
"indra",
"wigless",
"raucous",
"agential",
"observance",
"bawcock",
"serac",
"sent",
"anticlimax",
"solan",
"anesthesia",
"detestation",
"prussia",
"reservist",
"poets",
"chironomus",
"ocotillo",
"vakass",
"generator",
"fraternally",
"gymnura",
"convocation",
"copernicus",
"ruddy",
"galling",
"deportation",
"abrasive",
"cherubim",
"impassibility",
"seaport",
"absorption",
"gelignite",
"chipmunk",
"coprinaceae",
"biopsy",
"oryzomys",
"deparia",
"moneys",
"trop",
"silverware",
"cannae",
"cam",
"foenum",
"achene",
"approximatively",
"astrophyton",
"prance",
"inefficiently",
"noncolumned",
"alderman",
"amperage",
"anorectic",
"mendaciously",
"gager",
"pharaonic",
"taenia",
"nonacceptance",
"bursting",
"pretentiousness",
"tendentiousness",
"flabbergast",
"sigmoidoscope",
"goethes",
"cytotoxic",
"propets",
"intersection",
"spoils",
"corixidae",
"unsympathetic",
"untimely",
"overexcited",
"desensitized",
"sequestrate",
"guyot",
"diabolology",
"homemaker",
"fraction",
"bouncer",
"ptomaine",
"bicornute",
"fratercula",
"increment",
"streamliner",
"gratuitious",
"religieuse",
"leucocytozoan",
"athanasian",
"bradycardia",
"gems",
"chinch",
"necroscopic",
"throng",
"aerodynamic",
"bailiwick",
"dol",
"vilnius",
"macheath",
"multilateral",
"mildness",
"nonabsorbent",
"comradely",
"proprietorship",
"prostate",
"chorditis",
"process",
"prenticeship",
"preside",
"abhorrent",
"botaurus",
"eile",
"timeserver",
"opposing",
"master",
"podiceps",
"dijudication",
"bagpipes",
"trustfully",
"intensifying",
"firearm",
"anent",
"belladonna",
"alimentative",
"triplicity",
"immesh",
"pirogue",
"elect",
"reclining",
"brawling",
"aeromancy",
"addlehead",
"preconditioned",
"benefit",
"disputed",
"exchangeability",
"matchboard",
"escalator",
"hesitate",
"chorizema",
"attainment",
"falcade",
"improperly",
"seclusion",
"languages",
"unnecessarily",
"betel",
"eyelet",
"mannered",
"purism",
"rheumatology",
"diathermy",
"ascendancy",
"drunkenness",
"tenpin",
"victrola",
"angiosperm",
"drag",
"fungus",
"ciprofloxacin",
"sleight",
"iroquoian",
"yelp",
"heterocephalus",
"mouchard",
"cylindrical",
"azadirachtin",
"karelian",
"thirteenth",
"gorgonocephalus",
"battleax",
"antiadrenergic",
"gasp",
"psilocybin",
"betrayer",
"punitory",
"monkshood",
"mexico",
"baccarat",
"sagittiform",
"fascicule",
"morphologic",
"ambivalence",
"centner",
"psychopathic",
"bibliotic",
"donec",
"presto",
"eperon",
"pycnodysostosis",
"pee",
"utica",
"megilp",
"ponder",
"rabbitwood",
"desirableness",
"jonson",
"puffball",
"scape",
"aftergrowth",
"semiprecious",
"tallies",
"halchidhoma",
"forehanded",
"handshake",
"hylocichla",
"czarowitz",
"kera",
"sneer",
"inexpressile",
"wormwood",
"diamagnetism",
"rani",
"uninucleate",
"unclassifiable",
"crimination",
"meaning",
"astronaut",
"reek",
"leucothoe",
"therm",
"rumanian",
"enumerate",
"immaculate",
"modulus",
"frenzy",
"tromp",
"blattella",
"gutierrezia",
"hillbilly",
"decoy",
"fancifully",
"easy",
"demiurgus",
"scrutineer",
"argiopidae",
"saprolite",
"leprosy",
"accommodative",
"partitive",
"birthday",
"amniocentesis",
"mohammedan",
"parvovirus",
"inasmuch",
"ufa",
"testimonial",
"cognomen",
"ofiform",
"contractility",
"dwindle",
"arguer",
"maker",
"delator",
"unscrupulousness",
"gloominess",
"bauxite",
"tawniness",
"colleen",
"lawn",
"duties",
"malison",
"befoul",
"superhuman",
"preventative",
"ecumenic",
"melanitta",
"living",
"harare",
"artillery",
"parry",
"terebellidae",
"despised",
"philosophia",
"parody",
"gerbillus",
"encyclopedical",
"vivisection",
"misjudge",
"oration",
"hygiene",
"existential",
"prototherian",
"tumultuaary",
"hornmad",
"trenchant",
"valere",
"critical",
"zooplankton",
"displeasing",
"centrocercus",
"samiel",
"cornus",
"dunces",
"ravigote",
"cardamine",
"faux",
"gratingly",
"wholesome",
"imperativeness",
"camping",
"trave",
"urine",
"georgetown",
"glowering",
"selfinterested",
"coucicouci",
"zenana",
"impressionable",
"rage",
"lapster",
"monaurally",
"displease",
"manifesto",
"hispaniola",
"bulbar",
"casehardened",
"extremum",
"mascara",
"conglaciation",
"fellows",
"resinated",
"thickhead",
"showers",
"piety",
"brigantine",
"splatter",
"schoolchild",
"maybe",
"glossographer",
"scampi",
"hollowware",
"palmae",
"redistribution",
"architectonics",
"thinskinned",
"hosta",
"misrepresentation",
"spatchcock",
"reservation",
"ladylikeness",
"tale",
"unmechanical",
"unthankfulness",
"tachinidae",
"cornsmut",
"showjumping",
"specialist",
"buoyant",
"cairn",
"liquidate",
"entgtg",
"osier",
"pollucite",
"comfortableness",
"homogenize",
"aftercome",
"voltarian",
"hippie",
"chowchow",
"magnislat",
"noninterchangeable",
"scour",
"figurate",
"odalisque",
"photography",
"diverge",
"superfatted",
"cherished",
"transept",
"inhibition",
"olfactory",
"bewitch",
"bloodletting",
"aedile",
"christmas",
"assertive",
"endocrinology",
"brawn",
"semiopacous",
"synchronize",
"encore",
"thriftless",
"incus",
"grey",
"aniline",
"coronach",
"cote",
"plantago",
"stochastic",
"weimar",
"cayuga",
"lifegiving",
"antilocapra",
"stained",
"deaconess",
"bells",
"coolie",
"frondeur",
"monarchies",
"ouster",
"barnstormer",
"unjustifiably",
"myopia",
"radiology",
"stuccco",
"sidewinder",
"extremis",
"rei",
"disembarrassment",
"visceral",
"photoconductivity",
"birth",
"sciagraphy",
"habergeon",
"libra",
"cloudlike",
"premises",
"bombproof",
"fatiguing",
"blether",
"frippery",
"heterosexual",
"nonage",
"ribbonlike",
"infatuation",
"tsungli",
"rageful",
"ultimacy",
"admeasurement",
"rambutan",
"anime",
"superjunction",
"pseudopod",
"follies",
"stockman",
"spondee",
"yip",
"systole",
"stopper",
"blinkard",
"challah",
"blackquarter",
"perishable",
"curmudgeon",
"transpire",
"verecundiam",
"bordeaux",
"plagioclastic",
"selfabasement",
"confluxible",
"delabrement",
"knarl",
"dehort",
"calosoma",
"retrench",
"scandaleuse",
"obtrusive",
"nolition",
"bantu",
"prospect",
"janua",
"charcoal",
"sagittate",
"timbrel",
"flodden",
"complete",
"qc",
"pusillanimity",
"scrip",
"orthodontics",
"diethylstilbestrol",
"squamous",
"scaleless",
"gabonese",
"acknowledgment",
"pseudobombax",
"matrimony",
"tappet",
"thyrotoxic",
"campaigner",
"impolite",
"miscreance",
"cuterebridae",
"epode",
"crystallography",
"watered",
"platinum",
"neurasthenic",
"nonconductive",
"gestic",
"livy",
"saddled",
"rugby",
"radiance",
"bulbul",
"camino",
"stablish",
"pipra",
"shocking",
"affidavit",
"undeciphered",
"whale",
"tormentor",
"fricassee",
"torment",
"aequo",
"albulidae",
"auk",
"robes",
"mercenaria",
"undefiled",
"pika",
"clydesdale",
"hemorrhoid",
"hibernal",
"legato",
"baseborn",
"veins",
"beehive",
"prismatic",
"unutterable",
"overawed",
"scute",
"circa",
"clipper",
"querulous",
"daycare",
"balaclava",
"slopper",
"unwilling",
"accompany",
"slice",
"offset",
"barghest",
"catacorner",
"tisk",
"perpetuum",
"gluttonous",
"polygonaceae",
"hypsiglena",
"foram",
"fossilization",
"sprit",
"arcadian",
"forgettable",
"lakefront",
"permit",
"sharpen",
"curettage",
"escrow",
"mensal",
"concentrate",
"broomcorn",
"dasypus",
"chlamydosaurus",
"cayenne",
"hint",
"unregistered",
"agas",
"lithology",
"phonetism",
"airsick",
"horst",
"uncompelled",
"balkans",
"wonderment",
"baud",
"strained",
"endermic",
"dilapidation",
"overemotional",
"butuminous",
"odorament",
"spire",
"dipper",
"hemeralopia",
"turbination",
"food",
"coastguard",
"sinistrorse",
"flunky",
"unkindly",
"shakily",
"equatability",
"friars",
"gullery",
"obligatory",
"ingeniously",
"branta",
"curricle",
"lobularity",
"tendency",
"frequency",
"confidant",
"foliaceous",
"jampot",
"prestigious",
"tannery",
"succurrere",
"subalpine",
"svalbard",
"incipient",
"glaciation",
"vindicator",
"effortlessly",
"unthoughtfulness",
"campylotropous",
"nonrepresentational",
"permissin",
"propaganda",
"conscire",
"alliaceous",
"est",
"ambercolored",
"illumine",
"tautogolabrus",
"mniaceae",
"formulaic",
"factually",
"insectivorous",
"reckoning",
"vavasour",
"cypriniformes",
"paleocortical",
"enchilada",
"scorse",
"snivel",
"xenogenetic",
"comeliness",
"ay",
"tendril",
"littera",
"actinomycosis",
"spirits",
"gladii",
"lukes",
"epitheliod",
"inordinate",
"diagonalizable",
"montanan",
"ghastliness",
"sandarac",
"praised",
"bees",
"unmarked",
"slobber",
"midriff",
"hematuria",
"skiagraphy",
"connarus",
"myrsinaceae",
"apostleship",
"paramilitary",
"navarino",
"argentic",
"created",
"anything",
"melastoma",
"irreversibly",
"aphagia",
"rheum",
"or",
"dabster",
"nondeductible",
"libadist",
"avile",
"mouflon",
"nonextant",
"old",
"intensive",
"megahertz",
"dedicated",
"dumfounder",
"annexion",
"armor",
"lowercase",
"ongoing",
"pricks",
"embers",
"unrealistically",
"correspondent",
"chunk",
"gem",
"computational",
"blattodea",
"wellnatured",
"homicidal",
"plashy",
"coadjutancy",
"gummosis",
"dialectics",
"recursive",
"sphaeriaceae",
"maneuverer",
"averti",
"articulo",
"homophone",
"bouge",
"cerement",
"disqualification",
"hemerocallis",
"alewife",
"demonstrative",
"forefront",
"sarcastically",
"thecodontia",
"bamboo",
"legal",
"archil",
"antigen",
"colony",
"boom",
"corselet",
"dyes",
"necio",
"propagandism",
"exhilarate",
"incendiarism",
"pinnotheres",
"adjuc",
"unpersuasiveness",
"infomercial",
"memoria",
"cope",
"cumulonimbus",
"orders",
"wi",
"abduction",
"camelot",
"vaseline",
"precedence",
"annis",
"microscopy",
"isothermal",
"sweepstake",
"enantiomorph",
"sneering",
"weekly",
"econometric",
"picogram",
"rc",
"sardonic",
"caudally",
"believing",
"liar",
"liquefied",
"spahee",
"breath",
"pavo",
"plantal",
"elastic",
"litote",
"anastrophy",
"snuffy",
"solidified",
"nidus",
"abra",
"stockbroker",
"shillings",
"fiercely",
"insole",
"surviving",
"cuckoopint",
"mephobarbital",
"aleurone",
"compress",
"advisable",
"sink",
"inimitably",
"uncoated",
]
